% $$$   This codes builds the dynamic matrix A for small perturbations to
% $$$   the parabolic profile in the channel flow. This is
% $$$   done by discretizing the Orr-Sommerfeld/Squire equations
% $$$   by mean of chebychev collocation for one wave number pair (kx,kz).
% $$$   It makes use of the Matlab differentiation suite (here functions chebdif.m, cheb4c.m) 
% $$$   written by Weideman and Reddy.
% $$$   The velocity perturbation is expressed in the wall-normal velocity, 
% $$$   wall normal vorticity form.
% $$$ 
% $$$  It then compute for each time the maximum energy growth
% $$$ and corresponding initial condition q0 and optimal response at considered time qmax
% $$$ It then plot for each computed time the 5 maximum singular values corresponding to the five optimal growth
% $$$   You can view the corresponding initial condition and response by clicking on the desired growth
% $$$ 
% $$$   Code written by jerome hoepffner, PhD student at KTH, Sweden
% $$$   For question, bugs and comments, contact jerome@mech.kth.se

%%%% parameters
Re=1000;%reynolds number (based on channel half width)
N=60;%number of collocation points in wall normal direction
kx=0;%streamwise wave number
kz=1;%spanwise wave number
maxtime=500;%max time for envelope
n=50;%number of timepoint to compute
nsave=5;%number of singular values to be saved

%%%% differentiation matrices
[yvecT,DM] = chebdif(N+2,2);
yvec=yvecT(2:end-1);

%%%% the velocity profile
U.u  = 1-yvec.^2;                            
U.P = -2*yvec;                              
U.PP= -2*ones(size(yvec));

% implement homogeneous boundary conditions 
D2=DM(2:N+1,2:N+1,2);

% fourth derivative with clamped conditions
[y,D4]=cheb4c(N+2);

%%%% laplacian
I=eye(N);
k2=kx^2+kz^2;
delta=(D2-k2*I);
delta2=(D4-2*k2*D2+k2*k2*I);

%%%% compute dynamic matrix
LOS  = i* kx*diag(U.u)  * delta -i* kx*diag(U.PP)  - delta2 / Re ;
LC    = -i*kz*diag(U.P) ;
LSQ   =-i* kx*diag(U.u) + delta/Re;
A = [-delta\LOS    zeros(N,N);  LC  LSQ ];

%%%% compute energy measure matrix
IWT=INTweights(N+2,boxsize);%routine for collocation integration weigths
IW=IWT(2:end-1,2:end-1);iw=diag(IW);iwT=diag(IWT);
[Q,QT]=ENERmat(DM,IWT,k2,N);%routine for the energy measure matrix Q

%%%% compute envelope of maximum transient growth 
[U,S]=eig(A);S=diag(S);% compute eigenvalues
vec=linspace(0,maxtime,n);%time vector
[F,Finv]=sqrth(U'*Q*U); % factorize energy measure in exp coef
g=zeros(nsave,n);
q0=zeros(2*N,nsave,n);
qmax=zeros(2*N,nsave,n);
for ind=1:n;% loop over time
  time=vec(ind);
  [u,s,v]=svd(F*diag( exp( S*time ) )*Finv);  s=diag(s);%svd decomposition
  q0=U*Finv*v(:,1:nsave);%worst case initial condition
  qmax=U*Finv*u(:,1:nsave);% optimal response
  maxgrowth=s(1:nsave).^2; 
  memg(:,ind)=maxgrowth;
  memq0(:,:,ind)=q0;
  memqmax(:,:,ind)=qmax;
  fprintf('%% it %3.0f / %3.0f time %8.3f || %8.3f  %8.3f \n',ind,n,time,g(1,ind),maxgrowth(1));
end

%%%% plot the envelope and set callback
figure(1);
subplot(1,1,1);
collist='brmgybbbbbbbbbbb';% color list for first, second... singular values
com='figure(2);plotvec(memq0(:,$,@),yvec);title(''worst case initial condition'');figure(3);plotvec(memqmax(:,$,@),yvec);title(''optimal response'')';%the ploting command for a given growth
for ind=1:n;%loop on time
  for indsave=1:nsave;%loop on singular values for a given time
    han=plot(vec(ind),memg(indsave,ind),[collist(indsave) '+']);%plot the eigenvalue
    currentcom=replaceind(replaceind(com,'@',ind),'$',indsave);%change indices in the callback command
    set(han,'buttondownfcn',currentcom);%set the callback
    hold on
  end
end
ylabel('Max energy amplification');
xlabel('time')
hold off


